package product

import (
	"context"
	"fmt"
	"gitea.timerzz.com/kedaya_haitao/common/model/product"
	coach_client "gitea.timerzz.com/kedaya_haitao/common/pkg/coach-client"
	"github.com/golang/glog"
	"github.com/samber/lo"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
	"log/slog"
	"time"
)

type Controller struct {
	ctx        context.Context
	client     *coach_client.US
	updateTime time.Time // 上次抓取时间
	db         *gorm.DB
	calculates []productv1.CalculateProcess
	Option
}

func NewController(client *coach_client.US, db *gorm.DB) *Controller {
	ctl := &Controller{
		client: client,
		db:     db,
	}
	ctl.AutoMigrate()
	ctl.LoadOption()
	ctl.LoadCalculateProcess()
	return ctl
}

func (c *Controller) AutoMigrate() {
	if err := c.db.AutoMigrate(&productv1.Product{}, &productv1.HistoryPrice{}, &Option{}, &productv1.CalculateProcess{}); err != nil {
		panic(err)
	}
}
func (c *Controller) Run(ctx context.Context) {
	c.ctx = ctx
	ticker := time.NewTicker(c.Interval)
	if err := c.Crawl(); err != nil {
		slog.Error(err.Error())
	} else {
		slog.Info("抓取信息成功")
		c.updateTime = time.Now()
	}
	for {
		select {
		case <-ctx.Done():
			return
		case <-ticker.C:
			if err := c.Crawl(); err != nil {
				slog.Error(err.Error())
			} else {
				slog.Info("抓取信息成功")
				c.updateTime = time.Now()
			}
		}

	}
}

func (c *Controller) Crawl() error {
	slog.Info("开始抓取信息")
	for page, totalPage := 1, -1; page <= totalPage || totalPage == -1; page++ {
		resp, err := c.client.ViewAllBags(c.ctx, page)
		if err != nil {
			return fmt.Errorf("访问coach第%d页失败: %w", page, err)
		}
		totalPage = resp.PageData.TotalPages

		if err = c.saveRespData(resp.PageData.Products); err != nil {
			return fmt.Errorf("保存第%d页数据失败: %w", page, err)
		}
		glog.Infof("第%d页数据保存成功", page)
	}
	return nil
}

func (c *Controller) saveRespData(list []coach_client.Product) error {
	var products = make([]productv1.Product, 0, len(list))
	for _, resp := range list {
		for _, color := range resp.Colors {
			price, _ := lo.Find(resp.VariantsOnSale, func(item coach_client.Variant) bool {
				return item.Id == color.VgId
			})
			// 获取已经存的运费和得物价格
			var savedProduct productv1.Product
			c.db.Model(&savedProduct).Where("pid = ?", color.VgId).Select("freight", "dw_price").Scan(&savedProduct)
			if savedProduct.Freight == 0 {
				savedProduct.Freight = c.Freight
			}
			p := productv1.Product{
				UpdatedAt:     time.Now(),
				Name:          resp.Name,
				Pid:           color.VgId,
				Color:         color.Text,
				Link:          fmt.Sprintf("%s/%s", "https://www.coachoutlet.com", color.Url),
				Image:         color.Media.Thumbnail.Src,
				Orderable:     color.Orderable,
				DiscPercent:   price.Price.MarkdownDiscPercent,
				OriginalPrice: price.Price.Sales.Value,
				Freight:       savedProduct.Freight,
				ExchangeRate:  c.ExchangeRate,
				DWPrice:       savedProduct.DWPrice,
			}

			var calculate []productv1.CalculateProcess
			c.db.Model(&productv1.CalculateProcess{}).Find(&calculate, "pid = ? AND website = ?", color.VgId, productv1.WebSite_US_Coach_Outlet)
			p.CalCNY(append(calculate, c.calculates...))
			products = append(products, p)
		}
	}
	// 去重
	products = lo.UniqBy(products, func(p productv1.Product) string {
		return p.Pid
	})
	return c.db.Clauses(clause.OnConflict{
		Columns:   []clause.Column{{Name: "pid"}},
		DoUpdates: clause.AssignmentColumns([]string{"name", "color", "link", "orderable", "us_price", "cny_price", "cal_mark", "rate", "price_status", "disc_percent", "updated_at"}),
	}).Create(products).Error
}
